/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramWriter.java, v 1.1 31 ao�t 07 08:56:47 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.topology.joram;

import java.io.FileWriter;
import java.io.IOException;

import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.Topology;

/**
 * This class is able to create an a3server.xml files for the configuration of
 * Joram service.
 * 
 * @author Nicolas SALATGE - eBMWebSourcing
 * @author Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 */
public final class JoramWriter {

    /**
     * The class name of the Joram connection manager
     */
    private static final String CONNECTION_MANAGER_CLASS = "org.objectweb.joram.mom.proxies.ConnectionManager";

    /**
     * The class name of the TCP proxy service
     */
    private static final String TCP_PROXY_SERVICE_CLASS = "org.objectweb.joram.mom.proxies.tcp.TcpProxyService";

    /**
     * the topology object
     */
    private final Topology topology;

    /**
     * the default constructor
     * 
     * @param topology
     *            the topology
     */
    public JoramWriter(final Topology topology) {
        this.topology = topology;
    }

    /**
     * write the a3 server file
     * 
     * @param fileName
     *            the file name
     * @throws IOException
     *             impossible to create the file
     * @throws TopologyException
     *             impossible to analyze the topology
     */
    public void writeA3Server(final String fileName) throws IOException, TopologyException {
        final String buffer = this.generateA3Server();
        final FileWriter writer = new FileWriter(fileName);
        writer.write(buffer);
        writer.close();
    }

    /**
     * Generate the a3 server
     * 
     * @return the buffer of the a3server.xml file
     * @throws TopologyException
     *             impossible to analyze the topology
     */
    @SuppressWarnings("PMD.ConsecutiveLiteralAppends")
    public String generateA3Server() throws TopologyException {
        final StringBuffer res = new StringBuffer(550);
        res.append(this.generateHeader());
        res.append("<config>\n");
        res.append("\t<domain name=\"").append(this.topology.getDomain().getName()).append(
                "\" />\n");
        res.append(this.generateServerPart());
        res.append("</config>\n");
        return res.toString();
    }

    /**
     * generate the header.
     * 
     * @return the header
     */
    @SuppressWarnings("PMD.ConsecutiveLiteralAppends")
    public String generateHeader() {
        final StringBuffer res = new StringBuffer(500);
        res.append("<?xml version=\"1.0\"?>\n");
        res.append("<!-- <!DOCTYPE config SYSTEM \"a3config.dtd\"> -->\n");
        res.append("<!--\n");
        res.append("\tThis is the configuration file for JORAM.\n");
        res
                .append("\tIt MUST be used when running PEtALS with the static distributed server provided by JORAM.\n");
        res.append('\n');
        res
                .append("\tYou MUST dupplicate the values which are defined in the conf/server.properties\n");
        res.append("\tfile and set them in the current one.\n");
        res.append('\n');
        res
                .append("\tMore information on the PEtALS project webpage (http://petals.objectweb.org).\n");
        res.append("-->\n");
        return res.toString();
    }

    /**
     * Generate the JORAM servers part
     * 
     * @return the server part
     */
    @SuppressWarnings("PMD.ConsecutiveLiteralAppends")
    public String generateServerPart() {
        final StringBuffer res = new StringBuffer(200);

        for (final Subdomain subdomain : this.topology.getDomain().getSubDomain()) {

            for (final Container container : subdomain.getContainer()) {

                res.append("\t<server id=\"").append(container.getName()).append("\" name=\"")
                        .append(container.getName()).append("\" hostname=\"").append(
                                container.getHost()).append("\">\n");
                res.append("\t\t<network domain=\"").append(this.topology.getDomain().getName())
                        .append("\" port=\"").append(container.getJoramservice().getDomainPort())
                        .append("\"/>\n");
                res.append("\t\t<service class=\"").append(JoramWriter.CONNECTION_MANAGER_CLASS);

                if (container.getUser() != null) {
                    res.append("\" args=\"").append(container.getUser()).append(' ').append(
                            container.getPassword());
                }
                res.append("\"/>\n");

                if (container.getJoramservice().getTcpPort() != null) {
                    res.append("\t\t<service class=\"").append(JoramWriter.TCP_PROXY_SERVICE_CLASS)
                            .append("\" args=\"").append(container.getJoramservice().getTcpPort())
                            .append("\"/>\n");
                }
                res.append("\t</server>\n");
            }
        }
        return res.toString();
    }

}
